%define RAMSCREEN 0xB8000
%define SIZESCREEN 0xFA0; = 4000, nombres d'octets d'une page texte
%define SCREENLIM 0xFA0	; Si un octet >= SCREENLIM, il n'est pas dans 
						; la ram ecran.

;-----------------------------------------------------------------
; Entree:	
;	1er arg -> nombre de lignes a scroller (de 0 a 25)
;-----------------------------------------------------------------
scrollup:
	push ebp
	mov ebp,esp
	push eax
	push ebx
	push ecx
	push edx
	push esi
	push edi
	push ds
	push es

	xor edi,edi		; desination = 0xB8000:0
	mov eax,[ebp+8]		; nombre de lignes a scroller

; Decrementation de X
	cmp byte [ds:X],al		; si X < n
	jb .azero				; alors X->0
	sub byte [ds:X],al		; sinon X->X-n
	jmp .load
.azero:
	mov byte [ds:X],0

.load:
	push eax
	mov ax,0x20		; gdt_screen
	mov ds,ax
	mov es,ax
	pop eax

	push eax
	mov eax,0
	push eax
	call getpos		; calcul l'offset en ram de la ligne a scroller
	pop eax
	pop eax
	mov esi,edx		; esi -> offset en ram ecran a ramener a l'origine.
	push esi	; valeur qui servira pour le remplissage

	mov ecx,SIZESCREEN
	sub ecx,esi		; ecx -> nombre d'octets a recopier
	push ecx	; valeur qui servira pour le remplissage

	rep movsb		; transfere esi vers edi

	pop edi
	pop ecx
.remplissage:		; rempli le bas de l'ecran avec \0
	mov byte [edi],0
	inc edi	
	dec ecx
	jnz .remplissage
	
	pop es
	pop ds
	pop edi
	pop esi
	pop edx
	pop ecx
	pop ebx
	pop eax
	pop ebp
	ret
;-----------------------------------------------------------------
; Pour afficher une chaine de caracteres 
; Entree:
;	ds:esi  -> chaine terminee par \0
;-----------------------------------------------------------------
print:
	push ebp
	mov ebp,esp
	push eax
	push ebx
	push ecx
	push edx
	push esi
	push edi
	push ds
	push es

	xor eax,eax
	mov al,byte [X]		; position du curseur en argument
	push eax
	mov al,byte [Y]		
	push eax
	call getpos		; retourne dans edx position dans la ram
	pop eax
	pop eax
	mov edi,edx		; edi -> pointe sur offset en ram ecran du curseur

	mov ax,0x20
	mov es,ax

.afficher:
	lodsb
	cmp al,0	; teste si fin de chaine
	jz .fin			; si oui, fin.
	cmp al,0xa
	jz .cr_nl
	jmp .car
.cr_nl:
	mov byte [Y],79
	jmp .fin_afficher
.car:
	mov	byte [es:edi],al	; sinon, copie d'un caractere...
	inc edi
	mov byte al,[ds:XYattr]
	mov byte [es:edi],al	; ... et de l'attribut.
	inc edi
.fin_afficher:

.incrementeXY:
	add byte [Y],1			; on incremente Y
	cmp byte [Y],80			; si Y < 80
	jb .finIncrementeXY		; alors on affiche le caractere suivant
	mov byte [Y],0			; sinon, Y->0 et X->X+1
	add byte [X],1			; Note : si X>24, le scrolling mettra X a 24
.finIncrementeXY:

.testeLimites:
	xor eax,eax
	mov al,byte [X]		; position du curseur en argument
	push eax
	mov al,byte [Y]		
	push eax
	call getpos		; retourne dans edx position dans la ram
	pop eax
	pop eax
	mov edi,edx		; edi -> pointe sur offset en ram ecran du curseur

	cmp edi,SCREENLIM	; teste si on depasse les limites de l'ecran
	jb .afficher		; si non, on continue
	sub edi,160			; si oui, scrolle d'une ligne ...
	push byte 1	
	call scrollup
	pop eax			; on recupere la variable passe par le push
	jmp .afficher	; et on continue
	
.fin:
	pop es
	pop ds
	pop edi
	pop esi
	pop edx
	pop ecx
	pop ebx
	pop eax
	pop ebp
	ret

;-----------------------------------------------------------------
; Entree: 
;	1er arg -> x (0 a 24).
;	2e arg  -> y (0 a 79).
;-----------------------------------------------------------------
gotoxy:
	push ebp
	mov ebp,esp
	push eax

	mov eax,[ebp+8]		; recupere la valeur de y
	mov byte [Y],al	; mise a jour de 'curY'
	mov eax,[ebp+12]	; recupere la valeur de x
	mov byte [X],al	; mise a jour de 'curX'

	pop eax
	pop ebp
	ret

;-----------------------------------------------------------------
; Entree: 
;	1er arg -> x (0 a 24).
;	2e arg  -> y (0 a 79).
; Retour: 
;	edx -> offset a partir du debut de la ram ecran.
;-----------------------------------------------------------------
getpos:
	push ebp
	mov ebp,esp
	push ecx
	push ebx
	push eax

	mov ebx,[ebp+8]		; recupere la valeur de y
	shl ebx,1			; y*2

	mov eax,[ebp+12]	; recupere la valeur de x
	mov ecx,eax
	shl eax,7			; x*128
	shl ecx,5			; x*32
	add eax,ecx			; x*128+x*32 = x*160

	add eax,ebx			; eax -> x*160+y*2
	mov edx,eax

	pop eax
	pop ebx
	pop ecx
	pop ebp
	ret

;-----------------------------------------------------------------
; Entree: 
;	al -> attribut
;-----------------------------------------------------------------
color:
	mov byte [XYattr],al
	ret
;-----------------------------------------------------------------

;-----------------------------------------------------------------
; Pour afficher une chaine de caracteres 
; Entree:
;	1er argument -> caractere a afficher
;-----------------------------------------------------------------
putcar:
	push ebp
	mov ebp,esp
	push eax
	push ebx
	push ecx
	push edx
	push esi
	push edi
	push ds
	push es

	xor eax,eax
	mov al,byte [X]		; position du curseur en argument
	push eax
	mov al,byte [Y]		
	push eax
	call getpos		; retourne dans edx position dans la ram
	pop eax
	pop eax
	mov edi,edx		; edi -> pointe sur offset en ram ecran du curseur

	mov ax,0x20
	mov es,ax

.afficher:
	mov byte al,[ebp+8]		; On recupere le caractere dans al
	cmp al,0xA				; Teste si cr-nl
	jz .cr_nl
	jmp	.car
.cr_nl:					; Retour chariot + new line
	mov byte [Y],79		; On place le curseur en fin de ligne, il sera incremente
						; par la procedure .incrementeXY
	jmp .fin_afficher
.car:					; Caracter normal a afficher
	mov byte [es:edi],al	; Copie du caractere...
	inc edi
	mov byte al,[ds:XYattr]
	mov byte [es:edi],al	; ... et de l'attribut.
	inc edi
.fin_afficher:

	.incrementeXY:
	add byte [Y],1			; on incremente Y
	cmp byte [Y],80			; si Y < 80
	jb .finIncrementeXY		; alors on affiche le caractere suivant
	mov byte [Y],0			; sinon, Y->0 et X->X+1
	add byte [X],1			; Note : si X>24, le scrolling mettra X a 24
	cmp byte [X],25
	jb .finIncrementeXY
	push byte 1	
	call scrollup
	pop eax
	.finIncrementeXY:

.fin:
	pop es
	pop ds
	pop edi
	pop esi
	pop edx
	pop ecx
	pop ebx
	pop eax
	pop ebp
	ret
;-----------------------------------------------------------------


;-----------------------------------------------------------------
; clrscr
;	Synopsis: efface l'ecran
;-----------------------------------------------------------------
clrscr:
	push dword 25
	call scrollup
	pop eax
	ret

;-----------------------------------------------------------------
X: db 10
Y: db 0
XYattr: db 0x07
;-----------------------------------------------------------------

