; Make codes
%define LSHIFT	0x2A
%define RSHIFT	0x36
%define CTRL	0x1D
%define ALT		0x38

; Break codes
%define BRK_LSHIFT	0xAA
%define BRK_RSHIFT	0xB6
%define BRK_CTRL	0x9D
%define BRK_ALT		0xB8

;-----------------------------------------------------------------------------
; Routine d'interruption du clavier
;-----------------------------------------------------------------------------
kbd_interrupt:
	cli
	push eax
	push ebx

	xor eax,eax

.wait4buffer:		; On teste si le buffer de sortie est plein
	in al,0x64
	test byte al,1
	jz .wait4buffer	; Tant que le buffer est vide, on attend
					; REM : Il faudrait limiter le nb de loop !!!
	in al,0x60		; Lecture du scan-code
	cmp al,0x81		; On teste si c'est un breakcode
	jnb .breakcode

	.makecode:
		cmp al,LSHIFT
		jz .shift
		cmp al,RSHIFT
		jz .shift
		cmp al,CTRL
		jz .ctrl
		cmp al,0xE0
		jz .e0
		cmp al,0xE1
		jz .e1
		jmp .normal

		.normal:
			xor ebx,ebx
			dec al
			mov byte bl,[keytable+eax*4]
			push ebx
			call putcar
			pop ebx
			jmp .fin

		.shift:
			jmp .fin

		.ctrl:
			jmp .fin

		.e0:
		.e1:
			jmp .fin

	.breakcode:
		cmp al,BRK_LSHIFT
		jz .brkshift
		cmp al,BRK_RSHIFT
		jz .brkshift
		cmp al,BRK_CTRL
		jz .brkctrl
		jmp .fin

		.brkshift:
			jmp .fin

		.brkctrl:
			jmp .fin
	
.fin:
	mov al,0x20
	out 0x20,al		; EOI envoyee au PIC

	pop ebx
	pop eax
	sti

	ret

;-----------------------------------------------------------------------------
; Flags
e0flag: db 0
e1flag: db 0
sflag:	db 0	; shift
cflag:	db 0	; ctrl
aflag:	db 0	; alt

; Mappage des touches
; normal	shift	ctrl	alt		touche
keytable:
db 0x1B,	0x1B,	0x1B,	0x1B	; esc (0x01)
db '1',		'!',	'1',	'1'
db '2',		'@',	'2',	'2'
db '3',		'#',	'3',	'3'
db '4',		'$',	'4',	'4'
db '5',		'%',	'5',	'5'
db '6',		'^',	'6',	'6'
db '7',		'&',	'7',	'7'
db '8',		'*',	'8',	'8'
db '9',		'(',	'9',	'9'
db '0',		')',	'0',	'0'
db '-',		'_',	'-',	'-'
db '=', 	'+', 	'=', 	'='
db 0x08,	0x08,	0x7F,	0x08	; backspace
db 0x09,	0x09,	0x09,	0x09	; tab
db 'q',		'Q',	'q',	'q'
db 'w',		'W',	'w',	'w'
db 'e',		'E',	'e',	'e'
db 'r',		'R',	'r',	'r'
db 't',		'T',	't',	't'
db 'y',		'Y',	'y',	'y'
db 'u',		'U',	'u',	'u'
db 'i',		'I',	'i',	'i'
db 'o',		'O',	'o',	'o'
db 'p',		'P',	'p',	'p'
db '[',		'{',	'[',	'['
db ']',		'}',	']',	']'	
db 0x0A,	0x0A,	0x0A,	0x0A	; enter
db 0xFF,	0xFF,	0xFF,	0xFF	; ctrl
db 'a',		'A',	'a',	'a'	
db 's',		'S',	's',	's'	
db 'd',		'D',	'd',	'd'	
db 'f',		'F',	'f',	'f'	
db 'g',		'G',	'g',	'g'	
db 'h',		'H',	'h',	'h'	
db 'j',		'J',	'j',	'j'	
db 'k',		'K',	'k',	'k'	
db 'l',		'L',	'l',	'l'	
db ';',		':',	';',	';'	
db 0x27,	0x22,	0x27,	0x27	; '"
db '`',		'~',	'`',	'`'		; `~
db 0xFF,	0xFF,	0xFF,	0xFF	; Lshift (0x2a)
db '\',		'|',	'\',	'\'	
db 'z',		'Z',	'z',	'z'	
db 'x',		'X',	'x',	'x'	
db 'c',		'C',	'c',	'c'	
db 'v',		'V',	'v',	'v'	
db 'b',		'B',	'b',	'b'	
db 'n',		'N',	'n',	'n'	
db 'm',		'M',	'm',	'm'	
db 0x2C,	0x3C,	0x2C,	0x2C	;,<
db 0x2E,	0x3E,	0x2E,	0x2E	;.>
db 0x2F,	0x3F,	0x2F,	0x2F	;/?
db 0xFF,	0xFF,	0xFF,	0xFF	; Rshift (0x36)
db 0xFF,	0xFF,	0xFF,	0xFF	; (0x37)
db 0xFF,	0xFF,	0xFF,	0xFF	; (0x38)
db 0xFF,	0xFF,	0xFF,	0xFF	; (0x39)
db 0xFF,	0xFF,	0xFF,	0xFF	; (0x3a)
db 0xFF,	0xFF,	0xFF,	0xFF	; (0x3b)
db 0xFF,	0xFF,	0xFF,	0xFF	; (0x3c)
db 0xFF,	0xFF,	0xFF,	0xFF	; (0x3d)
db 0xFF,	0xFF,	0xFF,	0xFF	; (0x3e)
db 0xFF,	0xFF,	0xFF,	0xFF	; (0x3f)
db 0xFF,	0xFF,	0xFF,	0xFF	; (0x40)
db 0xFF,	0xFF,	0xFF,	0xFF	; (0x41)
db 0xFF,	0xFF,	0xFF,	0xFF	; (0x42)
db 0xFF,	0xFF,	0xFF,	0xFF	; (0x43)
db 0xFF,	0xFF,	0xFF,	0xFF	; (0x44)
db 0xFF,	0xFF,	0xFF,	0xFF	; (0x45)
db 0xFF,	0xFF,	0xFF,	0xFF	; (0x46)
db 0xFF,	0xFF,	0xFF,	0xFF	; (0x47)
db 0xFF,	0xFF,	0xFF,	0xFF	; (0x48)
db 0xFF,	0xFF,	0xFF,	0xFF	; (0x49)
db 0xFF,	0xFF,	0xFF,	0xFF	; (0x4a)
db 0xFF,	0xFF,	0xFF,	0xFF	; (0x4b)
db 0xFF,	0xFF,	0xFF,	0xFF	; (0x4c)
db 0xFF,	0xFF,	0xFF,	0xFF	; (0x4d)
db 0xFF,	0xFF,	0xFF,	0xFF	; (0x4e)
db 0xFF,	0xFF,	0xFF,	0xFF	; (0x4f)
db 0xFF,	0xFF,	0xFF,	0xFF	; (0x50)
db 0xFF,	0xFF,	0xFF,	0xFF	; (0x51)
db 0xFF,	0xFF,	0xFF,	0xFF	; (0x52)
db 0xFF,	0xFF,	0xFF,	0xFF	; (0x53)
db 0xFF,	0xFF,	0xFF,	0xFF	; (0x54)
db 0xFF,	0xFF,	0xFF,	0xFF	; (0x55)
db 0xFF,	0xFF,	0xFF,	0xFF	; (0x56)
db 0xFF,	0xFF,	0xFF,	0xFF	; (0x57)
db 0xFF,	0xFF,	0xFF,	0xFF	; (0x58)
db 0xFF,	0xFF,	0xFF,	0xFF	; (0x59)
db 0xFF,	0xFF,	0xFF,	0xFF	; (0x5a)
db 0xFF,	0xFF,	0xFF,	0xFF	; (0x5b)
db 0xFF,	0xFF,	0xFF,	0xFF	; (0x5c)
db 0xFF,	0xFF,	0xFF,	0xFF	; (0x5d)
db 0xFF,	0xFF,	0xFF,	0xFF	; (0x5e)
db 0xFF,	0xFF,	0xFF,	0xFF	; (0x5f)
db 0xFF,	0xFF,	0xFF,	0xFF	; (0x60)
db 0xFF,	0xFF,	0xFF,	0xFF	; (0x50)
db 0xFF,	0xFF,	0xFF,	0xFF
db 0xFF,	0xFF,	0xFF,	0xFF
db 0xFF,	0xFF,	0xFF,	0xFF
db 0xFF,	0xFF,	0xFF,	0xFF
db 0xFF,	0xFF,	0xFF,	0xFF
endkeytable:
