%define CS_ACCES	10011011b
%define DS_ACCES	10010011b
%define SS_ACCES	10010111b

%define GDTBASE		0x800
%define	IDTBASE		0x0

%define CS_LIM		0xFFFFF	; limite en blocs de 4 ko !
%define DS_LIM		0xFFFFF	; = 4 Go comme limite superieure.
%define SS_LIM		0x10	; = 0x10000 comme limite inferieure.
%define	STACKPTR	0x1FFFF	; adresse de depart de esp

%define MAXAPPELS	100


[BITS 32]
[ORG 0x1000]

jmp start

%include "GDT.INC"
%include "aff.asm"
%include "pic.asm"
%include "irq.asm"

start:
; initialisation de la gdt 
; descInit base(32),limite(20/32),acces(8),flags(4/8),adresse(16)
	descInit 0,CS_LIM,CS_ACCES,1101b,gdt_cs
	descInit 0,DS_LIM,DS_ACCES,1101b,gdt_ds
	descInit 0,SS_LIM,SS_ACCES,1101b,gdt_ss
	descInit 0xB8000,4000,DS_ACCES,0101b,gdt_screen		; ram ecran

; initialisation de gdtptr
	mov ax,gdtend	; calcule la limite de gdt
	mov bx,gdt
	sub ax,bx
	mov word [gdtptr],ax

	mov eax,GDTBASE		; adresse lineaire de gdt
	mov dword [gdtptr+2],eax

; initialisation de idtptr
	mov ax,idtend
	mov bx,idt
	sub ax,bx
	mov word [idtptr],ax

	mov eax,IDTBASE
	mov dword [idtptr+2],eax	

; recopie de la gdt a son adresse
	xor ecx,ecx
	xor esi,esi
	xor edi,edi
	mov	cx,word [gdtptr]	; charge la limite
	mov esi,gdt
	mov edi,GDTBASE
	rep movsb

; recopie de la idt a son adresse
	xor ecx,ecx
	xor esi,esi
	xor edi,edi
	mov	cx,word [idtptr]
	mov esi,idt
	mov edi,IDTBASE
	rep movsb

; chargement de la idt et de la nouvelle gdt
	lidt [idtptr]	; charge l'idt
	lgdt [gdtptr]	; charge la gdt

; initialisation des segments
	jmp next
next:
	mov ax,0x10		; segment de donne
	mov ds,ax
	mov es,ax
	mov fs,ax
	mov gs,ax
	mov ax,0x18		; pile
	mov ss,ax
	mov esp,STACKPTR

; un petit message
	call clrscr
	mov esi,msg00
	call print

; initialisation des interruptions
	cli
	call init_pic	; fichier pic.asm
	call init_irq	; fichier irq.asm
	sti

	mov esi,msg01
	call print

; initialisation des appels systemes
	gateInit 0x8,syscalls,TRAP_FLAG+PRESENT_F,0,GDTBASE+0x28		; selecteur,offset,wc,adresse

	mov esi,msg03
	call print


; KERNEL
	mov eax,2		; utilisation des appels systemes pour afficher un message
	mov esi,msg04
	int 0x80
end:
	jmp end
;-------------------------------------------------------------------


;-------------------------------------------------------------------
; ROUTINES APPELS SYSTEMES
; Entree:
;	eax -> numero de fonction
;	ebx,ecx,edx -> arguments
;	esi,edi -> autres arguments
;-------------------------------------------------------------------
syscalls:
	push ebx
	push ecx
	push edx
	call [systab+4*eax]
	pop ebx
	pop ecx
	pop edx
	iret
end_syscalls:

systab:
	dd scrollup
	dd gotoxy
	dd print
end_systab:


;-------------------------------------------------------------------
; DONNEES 
;-------------------------------------------------------------------

;-------------------------------------------------------------------
; table gdt
;-------------------------------------------------------------------
gdt:
gdt_null:			; 0x0
	dw 0,0,0,0
gdt_cs:				; 0x8
	dw 0,0,0,0
gdt_ds:				; 0x10
	dw 0,0,0,0
gdt_ss:				; 0x18
	dw 0,0,0,0
gdt_screen:			; 0x20
	dw 0,0,0,0
gdt_call_gate:		; 0x28
	dw 0,0,0,0
gdtend:

;-------------------------------------------------------------------
; table idt
;-------------------------------------------------------------------
idt:
	times 256 dw 0,0,0,0
idtend:

;-------------------------------------------------------------------
; pointeur sur gdt
;-------------------------------------------------------------------
gdtptr:
	dw	0x0000		; limite
	dd	0			; base

;-------------------------------------------------------------------
; pointeur sur idt
;-------------------------------------------------------------------
idtptr:
	dw	0x0000		; limite
	dd	0			; base

;-------------------------------------------------------------------
; MESSAGES
;-------------------------------------------------------------------
msg00: db 'mode protege',10,0
msg01: db 'interruptions etablies',10,0
msg02: db '.',0
msg03: db 'appels systemes etablis',10,0
msg04: db 'debug : ceci prouve que les appels systemes sont ok',10,0
